@@ -26,6 +26,7 @@ class BioerProtocol(Station):
             mm_tube_vertical_speed = 25,
             headroom_vol_from_tubes_to_pcr = 60,
             headroom_vol_from_tubes_to_dw = 10,
+            headroom_factor_mm: float = 1,
             control_well_positions = ['G12', 'H12'],
             pause_between_mastermix_and_elutes: bool = True,
             tube_block_model: str = 'opentrons_24_aluminumblock_nest_2ml_screwcap',
@@ -39,6 +40,7 @@ class BioerProtocol(Station):
             elution_air_gap = 10,
             final_mix_blow_out_height = -2,
             p300_tip_max_volume: float = 200,        # ul max volume managed by tips
+            mastermix_type: str = "ELITECH",
             ** kwargs):
 
         super(BioerProtocol, self).__init__(
@@ -61,11 +63,13 @@ class BioerProtocol(Station):
         self._mix_bottom_height = mix_bottom_height
         self._mm_plate_bottom_height = mm_plate_bottom_height
         self._dw_elutes_bottom_height = dw_elutes_bottom_height
+        self._mastermix_type: str = mastermix_type
         self._pk_tube_source = MultiTubeSource()
-        self._mm_tube_source = MultiTubeSource(vertical_speed=25)
+        self._mm_tube_source = MultiTubeSource(vertical_speed=15, name=self._mastermix_type)
         self._control_well_positions = control_well_positions
         self._pk_tube_bottom_height = pk_tube_bottom_height
         self._headroom_vol_from_tubes_to_pcr = headroom_vol_from_tubes_to_pcr
+        self._headroom_factor_mm = headroom_factor_mm
         self._headroom_vol_from_tubes_to_dw = headroom_vol_from_tubes_to_dw
         self._pause_between_mastermix_and_elutes = pause_between_mastermix_and_elutes
         self._tube_block_model = tube_block_model
@@ -112,7 +116,7 @@ class BioerProtocol(Station):
 
     @labware_loader(4, "_tube_block")
     def load_tube_block(self):
-        self._tube_block = self._ctx.load_labware(self._tube_block_model, '4', 'screw tube aluminum block for proteinase and mastermix')
+        self._tube_block = self._ctx.load_labware(self._tube_block_model, '4', 'aluminum block')
 
     @labware_loader(5, "_dests_plates")
     def load_dests_plate(self):
@@ -180,6 +184,10 @@ class BioerProtocol(Station):
             dests_pk_unique += d
         return dests_pk_unique
 
+    @property
+    def mastermix_tube_list(self):
+        return list(reversed(self._tube_block.wells()))
+
     @property
     def beads_destinations(self):
         return [row for plate in self._dests_plates for row in plate.rows()[0]][5::6]
@@ -290,41 +298,78 @@ class BioerProtocol(Station):
 
         done_col = 0
         source_el = source_plate[done_col:(done_col + len(source_plate))]
+
         for i, (s, d) in enumerate(zip(source_el, dests_sample_elute)):
+            controls = any([w in d for w in self.control_dests_wells])
+
+            if controls:
+                self.logger.info("Found controls in this row")
+
             samples_and_pipette = []
-            if any([w in d for w in self.control_dests_wells]):
-                # self.logger.info("Using single pipette")
+            if controls:
+                self.logger.info("We will use single pipetting")
                 for w, z in zip(s, d):
                     if z not in self.control_dests_wells:
-                        samples_and_pipette.append((w, z, self._s300))
+                        if z in self.sample_dests_wells:
+                            samples_and_pipette.append((w, z, self._s300))
+                        else:
+                            self.logger.info("Skipping well {} since it is not in samples list".format(z))
             else:
                 samples_and_pipette.append((s[0], d[0], self._p300))
 
             for j, (t, o, pipette) in enumerate(samples_and_pipette):
                 if self.run_stage("{} {}{}".format(stage, i+1, " {}/{}".format(j+1, len(samples_and_pipette)) if len(samples_and_pipette)>1 else "")):
-                    self.pick_up(pipette)
-                    if self._p300_fake_aspirate or self._s300_fake_aspirate:
-                        pipette.aspirate(1, t.top())
-                        pipette.dispense(1)
-                        if pipette == self._p300:
-                            self._p300_fake_aspirate = False
-                        elif pipette == self._s300:
-                            self._s300_fake_aspirate = False
-
-                    with MoveWithSpeed(pipette,
-                                       from_point=t.bottom(self._dw_elutes_bottom_height + 3),
-                                       to_point=t.bottom(self._dw_elutes_bottom_height),
-                                       speed=self._very_slow_vertical_speed, move_close=False):
-                        pipette.aspirate(self._elution_volume)
-                    pipette.air_gap(self._elution_air_gap)
-                    pipette.dispense(self._elution_air_gap, o.top())
-                    pipette.dispense(self._elution_volume, o.bottom(self._pcr_bottom_headroom_height))
-                    pipette.mix(5, 20, o.bottom(self._mix_bottom_height))
-                    pipette.blow_out(o.top(self._final_mix_blow_out_height))
-                    pipette.air_gap(self._elution_air_gap)
-                    self.drop(pipette)
+                    self.transfer_elute(t, o, pipette)
             done_col = done_col + len(source_plate)
 
+    def transfer_elute(self, source, dest, pipette):
+        self.pick_up(pipette)
+        if self._p300_fake_aspirate or self._s300_fake_aspirate:
+            pipette.aspirate(1, source.top())
+            pipette.dispense(1)
+            if pipette == self._p300:
+                self._p300_fake_aspirate = False
+            elif pipette == self._s300:
+                self._s300_fake_aspirate = False
+
+        with MoveWithSpeed(pipette,
+                           from_point=source.bottom(self._dw_elutes_bottom_height + 3),
+                           to_point=source.bottom(self._dw_elutes_bottom_height),
+                           speed=self._very_slow_vertical_speed, move_close=False):
+            pipette.aspirate(self._elution_volume)
+        pipette.air_gap(self._elution_air_gap)
+        pipette.dispense(self._elution_air_gap, dest.top())
+        pipette.dispense(self._elution_volume, dest.bottom(self._pcr_bottom_headroom_height))
+        pipette.mix(5, 20, dest.bottom(self._mix_bottom_height))
+        pipette.blow_out(dest.top(self._final_mix_blow_out_height))
+        pipette.air_gap(self._elution_air_gap)
+        self.drop(pipette)
+
+    @property
+    def mastermix_needed_volume(self):
+        """ Calculates the needed mastermix volume to ditribute to plate"""
+        volume_for_controls = len(self.control_wells_not_in_samples) * self._mm_volume
+        self.logger.info(
+            "Calculated {} ul for controls: {}".format(volume_for_controls, self.control_wells_not_in_samples))
+        volume_for_samples = self._mm_volume * self._num_samples
+        self.logger.info("Calcuated {} ul for samples".format(volume_for_samples))
+        return volume_for_samples + volume_for_controls
+
+    @property
+    def mastermix_total_volume(self):
+        """ Calculates the total volume of the mastermix: needed plus overheads """
+        return self.mastermix_needed_volume * self._headroom_factor_mm + self._headroom_vol_from_tubes_to_pcr
+
+    def calculate_and_load_mastermix_tubes(self):
+        num_tubes, vol_per_tube = uniform_divide(
+            self.mastermix_total_volume, self._mm_volume_tube)
+        available_volume = self.mastermix_needed_volume / num_tubes
+        available_volume += self._vol_mm_offset  # Adding volume for initial aspiration
+        assert vol_per_tube > available_volume, \
+            "Error in volume calculations: requested {}ul while total in tubes {}ul".format(available_volume,
+                                                                                            vol_per_tube)
+        [self._mm_tube_source.append_tube_with_vol(t, available_volume) for t in self.mastermix_tube_list[:num_tubes]]
+
     def body(self):
 
         num_dw = math.ceil(self._num_samples / self._max_sample_per_dw)
@@ -354,29 +399,13 @@ class BioerProtocol(Station):
 
 
         #tube_block_mm
-        volume_for_controls = len(self.control_wells_not_in_samples) * self._mm_volume
-        volume_for_samples = self._mm_volume * self._num_samples
-        volume_to_distribute_to_pcr_plate = volume_for_samples + volume_for_controls
-        num_tubes, vol_per_tube = uniform_divide(
-            volume_to_distribute_to_pcr_plate + self._headroom_vol_from_tubes_to_pcr, self._mm_volume_tube)
-        mm_tubes = self._tube_block.wells()[:num_tubes]
-        available_volume = volume_to_distribute_to_pcr_plate / len(mm_tubes)
-        if self._headroom_vol_from_tubes_to_pcr > 0:
-            if self._vol_mm_offset < (self._headroom_vol_from_tubes_to_pcr / len(mm_tubes)):
-                available_volume += self._vol_mm_offset
-        assert vol_per_tube > available_volume, \
-            "Error in volume calculations: requested {}ul while total in tubes {}ul".format(available_volume, vol_per_tube)
 
-        [self._mm_tube_source.append_tube_with_vol(t, available_volume) for t in self._tube_block.wells()[len(self._tube_block.wells())-num_tubes::]]
+        self.calculate_and_load_mastermix_tubes()
 
-        self.logger.info("We need {} tubes with {}ul of mastermix each in {}".format(num_tubes,
-                                                                                      vol_per_tube,
-                                                                                      self._mm_tube_source.locations_str))
         mmix_requirements = self.get_msg_format("load mm tubes",
-                                           num_tubes,
-                                           vol_per_tube,
-                                           self._mm_tube_source.locations_str)
-
+                                                self._mm_tube_source.num_tubes,
+                                                self._mm_tube_source)
+        self.logger.info(mmix_requirements)
         control_positions = self.get_msg_format("control positions",
                                                 self._control_well_positions)
 
@@ -396,7 +425,8 @@ class BioerProtocol(Station):
 
         #transfer mastermix
         if self._mastermix_phase:
-            self.pause(control_positions, home=False)
+            if self._control_well_positions:
+                self.pause(control_positions, home=False)
             self.pause(mmix_requirements, home=False)
             self.transfer_mastermix()
 
@@ -418,21 +448,45 @@ class BioerProtocol(Station):
 #             ** kwargs)
 
 class BioerPreparationToPcr(BioerProtocol):
-    def __init__(self, ** kwargs):
+    def __init__(self,
+                 transfer_elutes_phase: bool = True,
+                 ** kwargs):
         super(BioerPreparationToPcr, self).__init__(
             mastermix_phase = True,
-            transfer_elutes_phase = True,
+            transfer_elutes_phase = transfer_elutes_phase,
             ** kwargs)
 
 
 class BioerPreparationToPcrTechogenetics(BioerPreparationToPcr):
-    def __init__(self, ** kwargs):
+    _protocol_description = "Technogenetics mastermix and elutes distribution"
+    
+    def __init__(self,
+                 mm_volume=20,
+                 elution_volume=20,
+                 mm_volume_tube=1400,
+                 transfer_elutes_phase: bool = True,
+                 **kwargs):
         super(BioerPreparationToPcrTechogenetics, self).__init__(
-            mm_volume = 20,
-            elution_volume = 20,
-            mm_volume_tube = 1300,
+            mm_volume = mm_volume,
+            elution_volume = elution_volume,
+            mm_volume_tube = mm_volume_tube,
+            transfer_elutes_phase = transfer_elutes_phase,
+            mastermix_type = "KHB",
             **kwargs)
 
+
+class DistributeMastermixTechnogenetics(BioerPreparationToPcrTechogenetics):
+    _protocol_description = "Technogenetics mastermix distribution"
+
+    def __init__(self, **kwargs):
+        super(DistributeMastermixTechnogenetics, self).__init__(
+            transfer_elutes_phase=False,
+            headroom_factor_mm = 1.2,               # set 20% overhead for mmix transfer
+            headroom_vol_from_tubes_to_pcr = 0,     # set 0ul additive overhead for mmix transfer
+            control_well_positions = [],
+            **kwargs)
+
+
 # protocol for loading in Opentrons App or opentrons_simulate
 # =====================================
 logging.getLogger(BioerProtocol.__name__).setLevel(logging.INFO)
@@ -443,5 +497,5 @@ def run(ctx):
     return station.run(ctx)
 
 if __name__ == "__main__":
-    BioerProtocol(metadata={'apiLevel':'2.7'}).simulate()
+    DistributeMastermixTechnogenetics(metadata={'apiLevel':'2.7'}, num_samples=8).simulate()
 
