@@ -1,3 +1,4 @@
+import sys
 from json import JSONDecodeError
 from threading import Timer
 
@@ -9,7 +10,6 @@ from .sound_manager import SoundManager
 from .utils import ProtocolContextLoggingHandler, LocalWebServerLogger, DelayManager
 from .lights import Button, BlinkingLightHTTP, BlinkingLight
 from opentrons.protocol_api import ProtocolContext
-from opentrons.protocol_api.paired_instrument_context import PairedInstrumentContext
 from opentrons.types import Point
 import opentrons.commands.types
 from abc import ABCMeta, abstractmethod
@@ -56,11 +56,19 @@ def first_row(rack):
 
 class StationMeta(ABCMeta):
     def __new__(meta, name, bases, classdict):
+        logger = logging.getLogger()
         c = super(StationMeta, meta).__new__(meta, name, bases, classdict)
+        logger.info("Class {} looking for messages".format(name))
         try:
-            with open(os.path.join(os.path.dirname(module_path), 'msg', '{}.json'.format(name))) as f:
+            msg_file = os.path.join(os.path.dirname(sys.modules[c.__module__].__file__), 'msg', '{}.json'.format(name))
+            logger.debug("Opening file {}".format(msg_file))
+            with open(msg_file) as f:
                 c._messages = json.load(f)
         except FileNotFoundError:
+            logger.warning("No msg file found for class {}".format(name))
+            c._messages = {}
+        except AttributeError as e:
+            logger.error("Unexpected error loading msg file: {}".format(e))
             c._messages = {}
         return c
     
@@ -215,13 +223,16 @@ class Station(metaclass=StationMeta):
 
     @property
     def logger(self) -> logging.getLoggerClass():
-        if ((not hasattr(self, "_logger")) or self._logger is None) and self._ctx is not None:
+        if (not hasattr(self, "_logger")) or self._logger is None:
             self._logger = logging.getLogger(self.logger_name)
-            if self._logger.hasHandlers():
-                self._logger.handlers = []
-            self._logger.addHandler(ProtocolContextLoggingHandler(self._ctx))
         return self._logger
-    
+
+    def setup_protocolcontext_logger(self):
+        if self._ctx is not None:
+            if self.logger.hasHandlers():
+                self.logger.handlers = []
+            self.logger.addHandler(ProtocolContextLoggingHandler(self._ctx))
+
     def setup_opentrons_logger(self):
         stack_logger = logging.getLogger('opentrons')
         stack_logger.setLevel(self.logger.getEffectiveLevel())
@@ -286,7 +297,20 @@ class Station(metaclass=StationMeta):
     @property
     def num_cols(self) -> int:
         return math.ceil(self._num_samples/self._samples_per_col)
-    
+
+    def num_samples_in_row(self, row_number):
+        if row_number < 0 or row_number > 7:
+            raise Exception("Row number valid are 0-7: {}".format(row_number))
+
+        samples_in_row = list(range(0, self._num_samples))[row_number::8]
+
+        self.logger.debug("Samples in row {} are: {}".format(row_number, samples_in_row))
+        return len(samples_in_row)
+
+    @property
+    def num_samples_in_rows(self) -> [int]:
+        return [self.num_samples_in_row(r) for r in range(8)]
+
     @property
     def _tip_log_filepath(self) -> str:
         return os.path.join(self._tip_log_folder_path, self._tip_log_filename)
@@ -360,13 +384,7 @@ class Station(metaclass=StationMeta):
                             tiprack = t
                             break
             if tiprack is None:
-                if isinstance(pip, PairedInstrumentContext):
-                    # Paired pipette tiprack doesn't work with the for loop below
-                    # pipettes must be the same, so selecting the first tiprack
-                    tiprack = list(self._tipracks().keys())[0]
-                    self._logger.debug("Selectiong {} tiprack for paired pipette".format(tiprack))
-                else:
-                    raise RuntimeError("no tiprack associated to pipette {}".format(pip))
+                raise RuntimeError("no tiprack associated to pipette {}".format(pip))
             self._update_tip_log_count()
             if self._tip_log['count'][tiprack] == self._tip_log['max'][tiprack]:
                 # If empty, wait for refill
@@ -382,8 +400,7 @@ class Station(metaclass=StationMeta):
 
     @staticmethod
     def _get_pipette_num_channels(pip):
-        pips = pip._instruments.values() if isinstance(pip, PairedInstrumentContext) else [pip]
-        return sum([p.channels for p in pips])
+        return pip.channels
 
     def drop(self, pip):
         # Drop in the Fixed Trash (on 12) at different positions to avoid making a tall heap of tips
@@ -419,7 +436,7 @@ class Station(metaclass=StationMeta):
             self.msg = msg
             self.logger.log(level, self.msg)
         if home:
-            self._mov_manager.move_to_home()
+            self.home()
         if blink and not self._ctx.is_simulating():
             self._sound_manager.play("beep")
             lt = (BlinkingLightHTTP if self._dummy_lights else BlinkingLight)(self._ctx, t=blink_period/2)
@@ -485,6 +502,9 @@ class Station(metaclass=StationMeta):
         self.delay(self._delay_manager.get_remaining_delay(minutes=minutes_to_wait),
                    self.get_msg_format("waiting delay to elapse", minutes_to_wait))
 
+    def home(self):
+        self._mov_manager.move_to_home()
+
     def body(self):
         pass
 
@@ -494,6 +514,7 @@ class Station(metaclass=StationMeta):
     def run(self, ctx: ProtocolContext):
         self.status = "running"
         self._ctx = ctx
+        self.setup_protocolcontext_logger()
         self._mov_manager = MovementManager(self._ctx)
         self._button = (Button.dummy if self._dummy_lights else Button)(self._ctx, 'blue')
         if self._simulation_log_lws or not self._ctx.is_simulating():
