@@ -2,7 +2,7 @@ import json
 import os
 import time
 
-from opentrons.protocol_api import ProtocolContext, InstrumentContext, PairedInstrumentContext
+from opentrons.protocol_api import ProtocolContext, InstrumentContext
 from opentrons.protocol_api.labware import Well
 from opentrons.types import Location
 import logging
@@ -161,7 +161,7 @@ def uniform_divide(total: float, mpp: float) -> Tuple[int, float]:
 
 class WellWithVolume:
     """Class to make easy to calcualate and use the height of the liquid in a well"""
-    def __init__(self, well: Well, initial_vol: float, min_height: float = 0.5, headroom_height: float = 2.0):
+    def __init__(self, well: Well, initial_vol: float=0, min_height: float = 0.5, headroom_height: float = 2.0):
         """Class initialization
         :param total_vol: the total volume expected in the well
         :param min_height: optional, the minimum height
@@ -175,12 +175,15 @@ class WellWithVolume:
         self._min_height = min_height
         self._headroom_height = headroom_height
 
+    def __str__(self):
+        return "Well {} with volume {}".format(str(self._well), self._volume)
+
     @property
     def height(self) -> float:
         if self._well.diameter:
             remaining_height = self._volume / (math.pi * (self._well.diameter/2)**2)
         else:
-            remaining_height = self._volume / self._well.length**2
+            remaining_height = self._volume / (self._well.length * self._well.width)
 
         remaining_height -= self._headroom_height
         final_height = max(remaining_height, self._min_height)
@@ -201,11 +204,15 @@ class WellWithVolume:
         if self._volume < 0:
             self._volume = 0
 
+    @property
+    def well(self):
+        return self._well
+
 
 class MoveWithSpeed:
     """Class to make easy aspirate and dispense approaching with a defined speed.
     This should be useful with viscous liquid in order not to break the external meniscus on tip avoiding drops"""
-    def __init__(self, pip: Union[InstrumentContext, PairedInstrumentContext],
+    def __init__(self, pip: Union[InstrumentContext],
                  from_point: Location,
                  to_point: Location,
                  speed,
@@ -239,15 +246,18 @@ class MoveWithSpeed:
     def __enter__(self):
         self._logger.debug("Moving close. Force direct is: {}".format(self._force_direct))
         if self._move_close:
-            self._pip.move_to(self._from_point)
+            self._pip.move_to(self._from_point, publish=False)
             self._pip.move_to(self._to_point, force_direct=self._force_direct, speed=self._speed)
         else:
             self._pip.move_to(self._to_point)
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         self._logger.debug("Moving close. Force direct is: {}".format(self._force_direct))
-        self._pip.move_to(self._to_point)       # for safety, since we've a force_direct set on next move_to
-        self._pip.move_to(self._from_point, force_direct=self._force_direct, speed=self._speed if self._go_away else None)
+        self._pip.move_to(self._to_point, publish=False)       # for safety, since we've a force_direct set on next move_to
+        self._pip.move_to(self._from_point,
+                          force_direct=self._force_direct,
+                          speed=self._speed if self._go_away else None,
+                          publish=False)
 
 
 def get_labware_json_from_filename(filename: str = ""):
